@@ -44,24 +44,23 @@
                                           % +Error
             ssl_context/3,                % +Role, -Config, :Options
             ssl_add_certificate_key/4,    % +Config, +Cert, +Key, -Config
-            ssl_set_sni_hook/3,           % +Config, +Goal, -Config
+            ssl_set_options/3,            % +Config0, -Config, +Options
             ssl_negotiate/5,              % +Config, +PlainRead, +PlainWrite,
                                           %          -SSLRead,   -SSLWrite
             ssl_peer_certificate/2,       % +Stream, -Certificate
             ssl_peer_certificate_chain/2, % +Stream, -Certificates
-            ssl_session/2                 % +Stream, -Session
+            ssl_session/2,                % +Stream, -Session
+            ssl_secure_ciphers/1          % -Ciphers
           ]).
-:- use_module(library(socket)).
-:- use_module(library(error)).
 :- use_module(library(option)).
-:- use_module(library(debug)).
+:- use_module(library(settings)).
 :- use_module(library(crypto), []).     % force initialization of libcrypto
 
 :- use_foreign_library(foreign(ssl4pl)).
 
 :- meta_predicate
     ssl_context(+, -, :),
-    ssl_set_sni_hook(+, 3, -).
+    ssl_set_options(+, -, :).
 
 :- predicate_options(ssl_context/3, 3,
                      [ host(atom),
@@ -108,6 +107,10 @@ easily be used.
 @see library(socket), library(http/http_open), library(crypto)
 */
 
+:- setting(secure_ciphers, atom,
+           'EECDH+AESGCM:EDH+AESGCM:EECDH+AES256:EDH+AES256:EECDH+CHACHA20:EDH+CHACHA20',
+           "Default set of ciphers considered secure").
+
 %!  ssl_context(+Role, -SSL, :Options) is det.
 %
 %   Create an  SSL context.  The context  defines several properties
@@ -304,15 +307,21 @@ ssl_copy_context(SSL0, SSL) :-
     ssl_context(server, SSL, []),
     '_ssl_init_from_context'(SSL0, SSL).
 
-%!  ssl_set_sni_hook(+SSL0, :Goal, -SSL)
+%!  ssl_set_options(+SSL0, -SSL, +Options)
 %
-%   SSL is the same as SSL0, except  that the SNI hook of SSL is Goal.
-%   See  the   sni_hook(:Goal)  option   of  ssl_context/3   for  more
-%   information about this hook.
+%   SSL is the same as SSL0, except for the options specified in
+%   Options.  The following options are supported: close_notify/1,
+%   close_parent/1, host/1, peer_cert/1, ecdh_curve/1,
+%   min_protocol_version/1, max_protocol_version/1,
+%   disable_ssl_methods/1, sni_hook/1, cert_verify_hook/1. See
+%   ssl_context/3 for more information about these options. This
+%   predicate allows you to tweak existing SSL contexts, which can be
+%   useful in hooks when creating servers with the HTTP
+%   infrastructure.
 
-ssl_set_sni_hook(SSL0, Goal, SSL) :-
+ssl_set_options(SSL0, SSL, Options) :-
     ssl_copy_context(SSL0, SSL),
-    '_ssl_set_sni_hook'(SSL, Goal).
+    '_ssl_set_options'(SSL, Options).
 
 %!  ssl_negotiate(+SSL,
 %!                +PlainRead, +PlainWrite,
@@ -485,6 +494,28 @@ cert_accept_any(_SSL,
                 _ProblemCertificate, _AllCertificates, _FirstCertificate,
                 _Error).
 
+%!  ssl_secure_ciphers(-Ciphers:atom) is det.
+%
+%   Secure ciphers must guarantee forward secrecy, and must mitigate all
+%   known critical attacks. As of  2017,   using  the  following ciphers
+%   allows you to obtain grade A on https://www.ssllabs.com. For A+, you
+%   must also enable HTTP Strict Transport  Security (HSTS) by sending a
+%   suitable header field in replies.
+%
+%   Note that obsolete ciphers *must* be   disabled  to reliably prevent
+%   protocol downgrade attacks.
+%
+%   The Ciphers list is read from   the setting `ssl:secure_ciphers` and
+%   can be controlled using  set_setting/2   and  other  predicates from
+%   library(settings).
+%
+%   *BEWARE*: This list must be changed when attacks on these ciphers
+%             become known! Keep an eye on this setting and adapt it
+%             as necessary in the future.
+
+ssl_secure_ciphers(Cs) :-
+    setting(secure_ciphers, Cs).
+
 
                  /*******************************
                  *           MESSAGES           *
